/***********************************************************************************************************************
 * File Name    : hal_entry.c
 * Description  : Contains data structures and functions used in hal_entry.c.
 **********************************************************************************************************************/
/***********************************************************************************************************************
 * DISCLAIMER
 * This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products. No
 * other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
 * applicable laws, including copyright laws.
 * THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
 * THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED. TO THE MAXIMUM
 * EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES
 * SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS
 * SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability of
 * this software. By using this software, you agree to the additional terms and conditions found by accessing the
 * following link:
 * http://www.renesas.com/disclaimer
 *
 * Copyright (C) 2020 Renesas Electronics Corporation. All rights reserved.
 ***********************************************************************************************************************/
#include "hal_data.h"
#include <io_ep.h>
#include "common_utils.h"
#include "r_gpt.h"

FSP_CPP_HEADER
void R_BSP_WarmStart(bsp_warm_start_event_t event);
FSP_CPP_FOOTER

extern volatile bool g_sw1_press;
extern volatile bool g_sw2_press;
extern volatile uint8_t g_output_delay_overfow_flag;

/*******************************************************************************************************************//**
 * main() is generated by the RA Configuration editor and is used to generate threads if an RTOS is used.  This function
 * is called by main() when no RTOS is used.
 **********************************************************************************************************************/
void hal_entry(void)
{
   fsp_err_t err                           = FSP_SUCCESS;
   fsp_pack_version_t version              = {RESET_VALUE};
   uint16_t output_delay_s1                = 0;
   uint16_t output_delay_s2                = 0;
   /* version get API for FLEX pack information */
   R_FSP_VersionGet(&version);

   /* Example Project information printed on the RTT */
   APP_PRINT(BANNER_INFO, EP_VERSION, version.version_id_b.major, version.version_id_b.minor, version.version_id_b.patch);
   APP_PRINT(EP_INFO);

   /* Initialize External IRQ driver*/
   err = icu_init();
   /* Handle error */
   if(FSP_SUCCESS != err)
   {
       APP_ERR_PRINT("** ICU INIT FAILED ** \r\n");
       APP_ERR_TRAP(err);
   }

   /* Initialize GPT PWM Ref Timer driver*/
   err = gpt_ref_timer_PWM_init();
   /* Handle error */
   if(FSP_SUCCESS != err)
   {
       APP_ERR_PRINT("** GPT PWM Ref Timer INIT FAILED ** \r\n");
       APP_ERR_TRAP(err);
   }

   /* Initialize GPT PWM Output Delay Timer driver*/
   err = gpt_output_delay_timer_PWM_init();
   /* Handle error */
   if(FSP_SUCCESS != err)
   {
       APP_ERR_PRINT("** GPT PWM Output Delay Timer INIT FAILED ** \r\n");
       APP_ERR_TRAP(err);
   }

   /* Configure ODC and start GPT0 (Ref PWM) & GPT1 (PWM Output Delay) simultaneously */
   odc_init();

   /* Main loop */
   while (true)
   {
       /* Wait for the pushbutton S1 to to be pressed and the GPT1 (PWM Output Delay) overflow to occur at the same time */
       if((true == g_sw1_press) && ( 1 == g_output_delay_overfow_flag ))
       {
           /* Clear user pushbutton flag */
           g_sw1_press = false;

           /* Set output delays */
           /* for rising edge, pin A */
           err = R_GPT_PwmOutputDelaySet(&g_output_delay_timer_ctrl,
                                         GPT_PWM_OUTPUT_DELAY_EDGE_RISING,
                                         output_delay_s1,
                                         GPT_IO_PIN_GTIOCA);
           if(err != FSP_SUCCESS)
           {
               APP_ERR_TRAP(err);
           }

           /* for falling edge, pin B */
           err = R_GPT_PwmOutputDelaySet(&g_output_delay_timer_ctrl,
                                         GPT_PWM_OUTPUT_DELAY_EDGE_FALLING,
                                         output_delay_s1,
                                         GPT_IO_PIN_GTIOCB);
           if(err != FSP_SUCCESS)
           {
               APP_ERR_TRAP(err);
           }

           output_delay_s1++;
           if(output_delay_s1 >= 32)
           {
               output_delay_s1 = 0;
           }
       }
       /* Wait for the pushbutton S2 to to be pressed and the GPT1 (PWM Output Delay) overflow to occur at the same time */
       if((true == g_sw2_press) && (( 1 == g_output_delay_overfow_flag )))
       {
           /* Clear user pushbutton flag */
           g_sw2_press = false;

           /* Set output delays */
           /* for rising edge, pin B */
           err = R_GPT_PwmOutputDelaySet(&g_output_delay_timer_ctrl,
                                         GPT_PWM_OUTPUT_DELAY_EDGE_RISING,
                                         output_delay_s2,
                                         GPT_IO_PIN_GTIOCB);
           if(err != FSP_SUCCESS)
           {
               APP_ERR_TRAP(err);
           }

           /* for falling edge, pin A */
           err = R_GPT_PwmOutputDelaySet(&g_output_delay_timer_ctrl,
                                         GPT_PWM_OUTPUT_DELAY_EDGE_FALLING,
                                         output_delay_s2,
                                         GPT_IO_PIN_GTIOCA);
           if(err != FSP_SUCCESS)
           {
               APP_ERR_TRAP(err);
           }

           output_delay_s2++;
           if(output_delay_s2 >= 32)
           {
               output_delay_s2 = 0;
           }
       }
       /* Clear GPT1 (PWM Output Delay) overflow flag */
       g_output_delay_overfow_flag = 0;
   }

#if BSP_TZ_SECURE_BUILD
    /* Enter non-secure code */
    R_BSP_NonSecureEnter();
#endif
}

/*******************************************************************************************************************//**
 * This function is called at various points during the startup process.  This implementation uses the event that is
 * called right before main() to set up the pins.
 *
 * @param[in]  event    Where at in the start up process the code is currently at
 **********************************************************************************************************************/
void R_BSP_WarmStart(bsp_warm_start_event_t event)
{
    if (BSP_WARM_START_RESET == event)
    {
#if BSP_FEATURE_FLASH_LP_VERSION != 0

        /* Enable reading from data flash. */
        R_FACI_LP->DFLCTL = 1U;

        /* Would normally have to wait tDSTOP(6us) for data flash recovery. Placing the enable here, before clock and
         * C runtime initialization, should negate the need for a delay since the initialization will typically take more than 6us. */
#endif
    }

    if (BSP_WARM_START_POST_C == event)
    {
        /* C runtime environment and system clocks are setup. */

        /* Configure pins. */
        R_IOPORT_Open (&g_ioport_ctrl, g_ioport.p_cfg);
    }
}

#if BSP_TZ_SECURE_BUILD

BSP_CMSE_NONSECURE_ENTRY void template_nonsecure_callable ();

/* Trustzone Secure Projects require at least one nonsecure callable function in order to build (Remove this if it is not required to build). */
BSP_CMSE_NONSECURE_ENTRY void template_nonsecure_callable ()
{

}
#endif
